package jwars;

import java.util.Arrays;
import java.util.Iterator;

/**
 * This class contains the main method of JWars.
 */
public class Main {

  private static void printVersion(){
    System.out.println("This is "+Util.NAME_AND_VERSION);
    System.out.println(""+Util.GAME_NAME + " is an acronym for "+Util.FULL_GAME_NAME);
    System.out.println("Written by "+Util.AUTHORS);
  }

  private static void printHelp(){
    System.out.println("Parameters: [-ofhcndv] [prop_1=val_1]...[prop_n=val_n]");
    System.out.println();
    System.out.println("o: enable opengl pipeline");
    System.out.println("f: run in full-screen mode");
    System.out.println("h: print this help and exit");
    System.out.println("m: use Motif look-and-feel");
    System.out.println("n: use native look-and-feel");
    System.out.println("d: draw debug collision grid");
    System.out.println("a: bad ATI driver mode (disable window decorations)");
    System.out.println("v: print version information and exit");
    //System.out.println("s: print default server properties to standard output");
    //System.out.println("c: print default client properties to standard output");
    System.out.println("");
    System.out.println("Properties are stored in the format key=value.");
  }

  private static void parseChars(String arg){
    for(int i = 1; i < arg.length(); i++){
      switch(arg.charAt(i)){
        case 'o':
          System.out.println("Attempting to enable Java2D OpenGL pipeline");
          System.setProperty("sun.java2d.opengl", "True");
          break;
        case 'f':
          System.out.println("Enabling pseudo-fullscreen mode");
          Util.isFullScreen=true;
          break;
        case 'm':
          System.out.println("Setting Motif look-and-feel");
          try{
            javax.swing.UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
          } catch(Exception ex){
            System.out.println("Could not set CDE look-and-feel");
            ex.printStackTrace();
          }
          break;
        case 'a':
          Util.useSpecialATIHack = true;
          break;
        //case 's':
        //  Util.SV_PROPS.printDefaults(System.out);
        //  System.exit(0);
        //  break;
        //case 'c':
        //  Util.CL_PROPS.printDefaults(System.out);
        //  System.exit(0);
        //  break;
        case 'n':
          System.out.println("Setting native look-and-feel");
          try{
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.
                getSystemLookAndFeelClassName());
          } catch(Exception ex){
            System.out.println("Could not set native look-and-feel");
            ex.printStackTrace();
          }
          break;
        case 'd':
          System.out.println("Enabling debug grid rendering");
          Util.setDrawDebugGrids(true);
          break;
        case 'v':
          printVersion();
          System.exit(0);
          break;
        case 'h':
          printVersion();
          System.out.println();
          printHelp();
          System.exit(0);
          break;
        default:
          System.out.println("Unrecognized option: "+arg.charAt(i));
          System.out.println();
          printHelp();
          System.exit(-1);
      }
    }
  }

  private static void parseArgs(Iterator<String> args){
    while(args.hasNext()){
      String arg = args.next();
      String[] prop = arg.split("=");
      if(prop.length != 2){
        System.err.println("Illegal argument format. Must be on the form key=value");
        System.exit(-1);
      }
      if(Util.SV_PROPS.getProperty(prop[0]) != null){
        Util.SV_PROPS.setProperty(prop[0], prop[1]); //server properties
      } else if(Util.CL_PROPS.getProperty(prop[0]) != null){
        Util.CL_PROPS.setProperty(prop[0], prop[1]); //client properties
      } else {
        System.err.println("No such property: "+prop[0]);
        System.exit(0);
      }
    }
  }

  public static void main(String[] args){
    System.out.println("Initializing JWars");
    System.out.println("Currently running JVM version: "+
                       System.getProperty("java.runtime.version"));
    Iterator<String> argIterator = Arrays.asList(args).iterator();
    if(args.length > 0 && args[0].startsWith("-")){
      parseChars(argIterator.next());
    }
    parseArgs(argIterator);
    new WarMenu();
  }

}
