package jwars.control;

import java.util.ArrayList;
import java.util.List;

import jwars.model.Unit;
import jwars.model.WorldListener;

public class UnitSelectionModel implements WorldListener {

  private List<UnitSelectionListener> listeners = new ArrayList<UnitSelectionListener>();

  private Unit markedUnit = null;

  private void unitSelected(Unit newUnit, Unit oldUnit){
    for(UnitSelectionListener listener : listeners){
      listener.unitSelected(newUnit, oldUnit);
    }
  }

  public void markUnit(Unit u){
    if(u != null && !u.isSelectable()){
      return;
    }
    Unit oldSelection = markedUnit;
    markedUnit = u; //Do this *before* calling unitSelected so markedUnit()
    //behaves consistently
    unitSelected(u, oldSelection);
  }
  public void addListener(UnitSelectionListener listener){
    listeners.add(listener);
  }

  public Unit markedUnit(){
    return markedUnit;
  }

  public void removeListener(UnitSelectionListener listener){
    listeners.remove(listener);
  }

  public void unitNowOnBoard(Unit u){
  }

  public void unitNowOffBoard(Unit u){
    if(u == markedUnit){
      markUnit(null);
    }
  }

}
