package jwars.event;



/**
 * Thread safe implementation of EventQueue, which will execute its events
 * immediately after insertion into the queue.
 */
public class SynchronizedQueue extends EventQueue{

  public SynchronizedQueue(){
    super(0);
  }

  public synchronized void update(int time){
    super.update(time);
  }

  public synchronized void add(Event event){
    if(event.time == Event.notScheduled){
      //If event.time is set, then it is assumed that the addition of the event
      //happens because of e.g. redundant keyboard events, and in that case
      //the add request is ignored.
      //The super-class implementation would throw an Exception.
      super.add(event);
    }
  }

}
