package jwars.gui;

import java.util.ArrayList;

import jwars.model.map.CoordinateSystem;

/**
 * Class for managing the focus point on the map.
 * NOTE: should probably be moved to the GUI package
 *
 */
public class ViewPort{

  public final int PORT_WIDTH, PORT_HEIGHT;
  private int currentLocationX, currentLocationY;
  private final CoordinateSystem coords;

  private ArrayList<ViewPortListener> listeners =
      new ArrayList<ViewPortListener>();

  public ViewPort(CoordinateSystem coords, int startX, int startY,
                  int portWidth, int portHeight){
    this.coords = coords;
    this.PORT_WIDTH = portWidth;
    this.PORT_HEIGHT = portHeight;
    setLocation(startX, startY);
  }

  public int getCurrentX(){
    return currentLocationX;
  }

  public int getCurrentY(){
    return currentLocationY;
  }

  public void setLocation(int x, int y){

    x -= PORT_WIDTH / 2; //Compensation for origin in corner
    y -= PORT_HEIGHT / 2;

    x = (x < 0 ? 0 : x); //Restrict sign and magnitude of arguments
    y = (y < 0 ? 0 : y); //Nicer than 16 lines of if-else
    x = (x + PORT_WIDTH >= coords.WIDTH ? coords.WIDTH - PORT_WIDTH - 1 : x);
    y = (y + PORT_HEIGHT >= coords.HEIGHT ? coords.HEIGHT - PORT_HEIGHT - 1 : y);

    this.currentLocationX = x;
    this.currentLocationY = y;
    for(ViewPortListener v : listeners){
      v.reportLocationChange(x, y);
    }
  }

  public void nudge(int dx, int dy){
    setLocation(this.currentLocationX + dx, this.currentLocationY + dy);
  }

  public void addListener(ViewPortListener listener){
    listeners.add(listener);
  }

  public static interface ViewPortListener{

    public void reportLocationChange(int x, int y);

  }

}
