package jwars.model;



public class Coord{

  public int x, y;

  public Coord(){
    this(0,0);
  }

  public Coord(int x, int y){
    this.x = x;
    this.y = y;
  }

  public Coord(Coord c){
    this(c.x, c.y);
  }

  public void set(int x, int y){
    this.x = x;
    this.y = y;
  }

  public void set(Coord c){
    set(c.x, c.y);
  }

  public Coord plus(int x, int y){
    this.x += x;
    this.y += y;
    return this;
  }

  public Coord minus(int x, int y){
    return plus(-x,-y);
  }

  public Coord plus(Coord c){
    return plus(c.x, c.y);
  }

  public Coord minus(Coord c){
    return minus(c.x, c.y);
  }

  public Coord rotate(double theta){
    int oldX = x;
    double cos = Math.cos(theta);
    double sin = Math.sin(theta);
    x = (int)(x*cos - y*sin);
    y = (int)(oldX*sin + y*cos);
    return this;
  }

  public Coord times(double scale){
    x *= scale;
    y *= scale;
    return this;
  }

  /*public Coord rotate(Coord other, double theta){
    return minus(other).rotate(theta).plus(other);
  }*/

  public int length(){
    return dist(0,0);
  }

  public int dist(int x, int y){
    return (int)Math.hypot(this.x - x, this.y - y);
  }

  public int dist(Coord other){
    return dist(other.x, other.y);
  }

  public String toString(){
    return super.toString() + " [x="+x+"][y="+y+"]";
  }

  public boolean equals(Object o){
    if (!(o instanceof Coord))
      return false;
    Coord c = (Coord)o;
    return c.x == x && c.y == y;
  }

  public static void main(String[] args){
    Coord t1 = new Coord(2,1).times(50);
    System.out.println(""+t1);
    System.out.println(""+t1.rotate(-Math.PI/2));
    Coord temp = new Coord(7,17).times(37).rotate(Math.PI/2).plus(37,17);
    System.out.println(""+temp);
  }

}
