package jwars.model.map.generator;



/**
 * <p>q w</p>
 *
 * <p>q w</p>
 *
 * <p>e r</p>
 *
 * <p>r t</p> y u i o
 */
public class TerrainGenerator{

  private final HeightGenerator generator;
  public final int WIDTH;
  public final int HEIGHT;

  public TerrainGenerator(int width, int height, long seed){
    int sizeBits;
    //Make sure that the generated map is at least as large as width * height
    for(sizeBits = 0; (1<<sizeBits < width) || (1<<sizeBits < height); sizeBits++);

    generator = new HeightGenerator(seed, sizeBits);
    this.WIDTH = width;
    this.HEIGHT = height;

  }

  public FloatBuffer generateTreeBuffer(float forestation){
    FloatBuffer forestThickness = generator.generateHeights().crop(0,0,WIDTH, HEIGHT);
    //Ensure that thickness is measured between 0 and 1:
    forestThickness.scaleToFit(0,1);
    //Cut away thin areas to make growth sensibly sparse:
    forestThickness.cutOff(1-forestation);
    //Scale the numbers to work in a discrete model
    //(add one because casting would otherwise eliminate one level)
    return forestThickness;
  }

  /*public int[] generateTrees(float forestation, int discreteDensityCount){
    return generateTrees(forestation, discreteDensityCount, 0);
  }*/

  public FloatBuffer generateTrees(float forestation, int discreteDensityCount, int smoothification){
    FloatBuffer forestThickness = generateTreeBuffer(forestation);
    for(int i = 0; i < smoothification; i++){
      forestThickness.smoothify();
    }
    //forestThickness.display();
    forestThickness.scaleToFitStrict(0,discreteDensityCount-1);
    //System.out.println("thickness min "+forestThickness.minValue()+" max "+forestThickness.maxValue());
    //float[] forestationValues = forestThickness.getBuffer();
    //int[] discreteForestationValues = new int[forestationValues.length];
    //Represent forestation as integers:
    //for(int i = 0; i < forestationValues.length; i++){
    //  discreteForestationValues[i] = (int)forestationValues[i];
    //}
    //return discreteForestationValues;
    return forestThickness;
  }

}
