package jwars.model.structures;

import jwars.model.Coord;

public class BluePrint{

  private Coord[] convexHull;
  private boolean massive;
  private boolean moveable;
  private boolean isPolygon;
  private int radius;
  //private int entrenchment;
  //private Coord center = new Coord();
  //private int radius;


  // used for creating terrainobjects with special collision profile
  // determined by the given coordinates for creating a polygon
  public BluePrint(Coord[] fixPoints, boolean massive){
    this.massive = massive;
    moveable = false;
    isPolygon = true;
    // entrenchment = 1;
    //radius = 0;
    //int centerx = 0;
    //int centery = 0;
    this.convexHull = fixPoints;
    setRadius(fixPoints);

  }

  // used for creating terrainobject with a circular collision profile
  // the circular profile is transferred to a convexhull with 8 points
  public BluePrint(boolean massive, int radius){
    this.massive = massive;
    this.radius = radius;
    moveable = false;
    isPolygon = false;

    setFixPoints(radius);
  }

  private void setFixPoints(int radius){
    int le = (int)(radius*1.2);
    double len = Math.sqrt((double)(le)*(double)(le)/2);
    convexHull = new Coord[8];
    convexHull[0] = new Coord(le, 0);
    convexHull[1] = new Coord((int)(len), (int)(len));
    convexHull[2] = new Coord(0, le);
    convexHull[3] = new Coord(-(int)(len), (int)(len));
    convexHull[4] = new Coord(-le, 0);
    convexHull[5] = new Coord(-(int)(len), -(int)(len));
    convexHull[6] = new Coord(0, -le);
    convexHull[7] = new Coord((int)(len), -(int)(len));
  }

  private void setRadius(Coord[] corners){
    //System.out.println("corners "+java.util.Arrays.toString(corners));
    //System.out.println("center "+center);
    for(int i = 0; i < corners.length; i++){
      int distance = (int)Math.hypot(corners[i].x, corners[i].y);
      if(distance > radius){
        radius = distance;
        //radius = dist(p[i], c);
      }
    }
    //System.out.println("Setting radius "+(radius/WarModel.WORLD.meter));
  }

  public Coord[] createStructure(){
    //Coord[] newCoords = new Coord[fixPoints.length];
    //System.arraycopy(fixPoints, 0, newCoords, 0, newCoords.length);
    //System.out.println(""+java.util.Arrays.toString(fixPoints));
    //return newCoords;
    return duplicate(convexHull);
  }

  private Coord[] duplicate(Coord[] c){
    Coord[] clone = new Coord[c.length];
    for(int i = 0; i < c.length; i++){
      clone[i] = new Coord(c[i]);
    }
    return clone;
  }

  /* private float sqr(int x){
     return(float)x * x;
   }*/
  public boolean getMassive(){
    return massive;
  }

  public boolean getMoveable(){
    return moveable;
  }

  public boolean getisPolygon(){
    return isPolygon;
  }

  public int getRadius(){
    return radius;
  }

}
