package jwars.model.structures;

import java.awt.Color;
import java.awt.Graphics2D;

import jwars.gui.units.Sprite;
import jwars.model.collision.Collider;
import jwars.model.collision.CollisionProperties;
import jwars.model.Coord;
import jwars.model.WarModel;

public class PolygonStructure
    implements Collider{

  //private BluePrint bp;
  private Coord[] structPoints;
  private int radius;
  private Coord location = new Coord();
  private PolygonStructureSprite sprite;
  private CollisionProperties collisionProps;

  private final static Color shadow = new Color(0f, 0f, 0f, 0.3f);
  private final static int shadowWidth = 3;
  private final static int shadowHeight = 4;

  public PolygonStructure(BluePrint bluePrint, double angle, int x, int y){

    //bp = bluep;
    radius = 0;
    collisionProps = new CollisionProperties(this, bluePrint.getMassive(),
        bluePrint.getMoveable());

    this.location.set(x, y);
    for(Coord c : structPoints = bluePrint.createStructure()){
      c.rotate(angle).plus(location);
    }

    setRadius(structPoints, location);
    sprite = new PolygonStructureSprite(getRadius());

  }

  public CollisionProperties getCollisionProps(){
    return collisionProps;
  }

  private void setRadius(Coord[] corners, Coord center){
    //System.out.println("corners "+java.util.Arrays.toString(corners));
    //System.out.println("center "+center);
    for(int i = 0; i < corners.length; i++){
      int distance = corners[i].dist(center);
      if(distance > radius){
        radius = distance;
        //radius = dist(p[i], c);
      }
    }
    //System.out.println("Setting radius "+(radius/WarModel.WORLD.meter));
  }

  private int dist(Coord a, Coord b){
    return(int)Math.sqrt(sqr(a.x - b.x) + sqr(a.y - b.y));
  }

  private float sqr(int x){
    return(float)x * x;
  }

  public int getX(){
    return location.x;
  }

  public int getY(){
    return location.y;
  }

  public int getRadius(){
    return radius;
  }

  public Sprite getSprite(){
    return sprite;
  }

  public void collisionDetected(Collider other){
  }

  public String toString(){
    return super.toString() + "[x=" + getX() + "][y=" + getY() + "][r=" +
        getRadius() + "]";
  }

  private class PolygonStructureSprite
      extends Sprite{

    //Polygon points = new Polygon();
    int[] xPoints, yPoints;

    public PolygonStructureSprite(int radius){
      super(radius);
      xPoints = new int[structPoints.length];
      yPoints = new int[structPoints.length];
      for(int i = 0; i < structPoints.length; i++){
        xPoints[i] = structPoints[i].x - location.x;
        yPoints[i] = structPoints[i].y - location.y;
      }
      WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS,
          xPoints);
      WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS,
          yPoints);
    }

    public void draw(Graphics2D g2d, int x, int y){
      g2d.translate(x - shadowWidth, y + shadowHeight);
      g2d.setColor(Color.black);
      g2d.fillPolygon(xPoints, yPoints, xPoints.length);
      g2d.translate(shadowWidth, -shadowHeight);
      g2d.setColor(Color.darkGray);
      g2d.fillPolygon(xPoints, yPoints, xPoints.length);
    }

  }

}
