package jwars.model.units;

import java.util.Iterator;

import jwars.event.dynamic.DEvent;
import jwars.data.ArtilleryCategory;
import jwars.model.Unit;
import jwars.model.ai.Attackable;
import jwars.model.ai.ListeningAI;
import jwars.model.ai.UnitAI;
import jwars.model.ai.impl.BasicAttacker;
import jwars.model.weapons.ReloadListener;
import jwars.model.weapons.Weapon;
import jwars.model.weapons.WeaponModel;

public class Artillery extends Unit {

  private UnitAI ai;
  private WeaponModel weaponry;

  public Artillery(ArtilleryCategory cat){
    super(cat.name, 0);
    ArtilleryAI ai = new ArtilleryAI();
    this.ai = ai;
    setAI(ai);
    weaponry = new WeaponModel(ai, cat.weaponry);
    for(Weapon w : weaponry.getWeaponList()){
      w.setImpactDistCalculator(new Weapon.DefaultImpactDistCalculator(w){
        public float impactDistFromTarget(int distance){
          return super.impactDistFromTarget((int)weapon.CAT.effectiveRange);
        }
      });
    }
  }

  public void update(){
    if(ai.isFiring() && weaponry.canFire()){
      weaponry.fire(getX(), getY(), location.x, location.y);
    }
  }

  private class ArtilleryAI implements ReloadListener, UnitAI {

    private BasicAttacker ai = new BasicAttacker(Artillery.this);

    public void reportFinished(Object source){
    }

    public void reportFailed(Object source){
    }

    public void orderAttack(Attackable target){
      //ai.orderAttack(target);
    }

    public void orderAttack(int x, int y, int radius){
      Artillery.this.location.set(x,y);
      ai.orderAttack(x,y,radius);
    }

    public void orderHoldFire(){
      ai.orderHoldFire();
    }

    public boolean isFiring(){
      return ai.isFiring();
    }

    public Attackable getTarget(){
      return ai.getTarget();
    }

    public Iterator<jwars.model.ai.Position> iterator(){
      return UnitAI.PASSIVE_AI.iterator();
    }

    public Unit owner(){
      return Artillery.this;
    }

    public void orderMove(ListeningAI source, int x, int y){
    }

    public void appendMove(ListeningAI source, int x, int y){
    }

    public void prependMove(ListeningAI source, int x, int y){
    }

    public void orderHalt(ListeningAI source){
    }

    public void setDestination(int x, int y){
    }

    public boolean isMoving(){
      return false;
    }

    public boolean reportReloaded(Weapon weapon){
      return false;
    }

  }

}
