package jwars.model.units;

import java.awt.Color;
import java.awt.Graphics2D;

import jwars.data.InfantryCategory;
import jwars.gui.units.Sprite;
import jwars.model.Moveable;
import jwars.model.WarModel;
import jwars.model.World;
import jwars.model.weapons.ShellType;
import jwars.model.weapons.Weapon;
import jwars.model.weapons.ArmourModel;
import jwars.model.weapons.InfantryArmourModel;

public class InfantrySquad extends Moveable{

  private final static Color camo = Color.GREEN.darker().darker().darker();
  private final static Color feldGrau = Color.darkGray;

  private final static int[] xLocations = {0, -4, 5, 1, -8,  6,  -7,  8,   0, 7, 10, -4};
  private final static int[] yLocations = {0, -6, 3, 7, -2, -6,  7,  -2, -10, 8,  3,  9};

  private int strength;
  protected final InfantryCategory CAT;

  private Color uniformColor;
  private final ArmourModel armourModel;
  //This weapon is considered issued to every member of the squad.
  private Weapon primaryWeapon = null;

  public InfantrySquad(InfantryCategory cat){
    super(cat, false);
    this.CAT = cat;
    if(cat.squadSize > xLocations.length){
      throw new IllegalArgumentException("Illegal squad size: "+cat.squadSize+". Maximum allowed: "+xLocations.length);
    }
    this.strength = cat.squadSize;
    this.armourModel = new InfantryArmourModel(this);
    setSprite(new DefaultSprite());
    java.util.List<Weapon> weapons = this.weaponry.getWeaponList();
    if(!weapons.isEmpty()){
      Weapon w = weapons.get(0);
      if(w.CAT.type == ShellType.bullet){
        primaryWeapon = w;
        primaryWeapon.setSalvoSize(strength);
      }
    }
  }

  public void addToWorld(int x, int y, boolean addChildren){
    super.addToWorld(x, y, addChildren);
    if(this.subordinateOf(Teams.SOVIET_UNION)){
      uniformColor = camo;
    } else if(this.subordinateOf(Teams.GERMANY)){
      uniformColor = feldGrau;
    } else {
      uniformColor = Color.cyan;
    }
  }

  public int getStrength(){
    return strength;
  }

  public void kill(int men){
    strength-=men;
    this.dirtify();
    if(strength <= 0){
      strength = 0;
      destroyLater();
    } else if(primaryWeapon != null){
      primaryWeapon.setSalvoSize(strength);
    }
  }

  public ArmourModel getArmourModel(){
    return this.armourModel;
  }

  private class DefaultSprite extends Sprite{

    public DefaultSprite(){
      super(WarModel.WORLD.MAIN_COORDS.transformToCoords(WarModel.PIXEL_COORDS, CAT.radius));
    }

    public void draw(Graphics2D g, int x, int y){
      for(int i = 0; i < strength; i++){
        drawSoldier(g, x+xLocations[i], y+yLocations[i]);
      }
    }

    private void drawSoldier(Graphics2D g, int x, int y){
      g.setColor(uniformColor);
      g.fillRect(x-1, y, 2, 4); //Body
      g.drawLine(x, y-2, x, y-2); //Helmet
      g.setColor(Color.yellow);
      g.drawLine(x, y-1, x, y-1); //Head
      g.setColor(Color.black);
      g.drawLine(x-3, y, x+1, y+2); //Gun
    }

  }

}
