package jwars.model.vision;

import jwars.model.map.TileMap;

import jwars.synch.Timer;
import jwars.event.Event;
import jwars.model.map.CoordinateSystem;
import jwars.event.EventQueue;
import jwars.model.World;
import jwars.test.LinkList;

public class ObservationEnvironment{

  private int delay;
  private EventQueue queue;
  final CoordinateSystem mainCoords;
  final TileMap<ObserverTile> obsMap;
  public final int maxVisibility;
  public final int relaxationTime;
  final Timer timer;

  //private int maxVisInTiles;

  public ObservationEnvironment(Timer timer, CoordinateSystem coords, int checkDelay, int relaxationTime, int maxVisibility){
    this.timer = timer;
    this.mainCoords = coords;
    this.delay = checkDelay;
    this.queue = new EventQueue(delay);
    this.relaxationTime = relaxationTime;
    this.maxVisibility = maxVisibility;
    //Find reasonable tilemap size in terms of visibility
    int obsMapWidth = coords.WIDTH / maxVisibility + 1;
    int obsMapHeight = coords.HEIGHT / maxVisibility + 1;
    //int scale = Math.max
    //this.obsMap = null;
    ObserverTile[][] tiles = new ObserverTile[obsMapWidth][obsMapHeight];
    for(int i = 0; i < tiles.length; i++){
      for(int j = 0; j < tiles[i].length; j++){
        tiles[i][j] = new ObserverTile();
      }
    }
    this.obsMap = new TileMap<ObserverTile>(coords, tiles);
    //this.maxVisInTiles = mainCoords.transformToCoords(obsMap, maxVisibility);
    jwars.model.WarModel.QUEUES.register(queue); //ugly static reference
  }

  public void register(ObserverModel obs){
    if(obs.env != this){
      throw new RuntimeException("ObserverModel not associated with this ObservationEnvironment");
    }
    //obs.env = this;
    //int obsMapX = mainCoords.transformToCoords(obsMap, obs.body.getX());
    //int obsMapY = mainCoords.transformToCoords(obsMap, obs.body.getY());
    ObserverTile tile = obsMap.getTileAt(obs.subject.getX(), obs.subject.getY());
    tile.addLast(obs); //Observer now on obs map
    obs.event.enqueue(); //Observer will now be polled regularly for vision
  }

  public void traverseSurroundingTiles(ObserverModel obs, int dt){
    int x = obs.subject.getX();
    int y = obs.subject.getY();
    int minX = mainCoords.transformToCoords(obsMap, x-maxVisibility);
    int maxX = mainCoords.transformToCoords(obsMap, x+maxVisibility);
    int minY = mainCoords.transformToCoords(obsMap, y-maxVisibility);
    int maxY = mainCoords.transformToCoords(obsMap, y+maxVisibility);

    /*double dist;
    if((dist=Math.max(maxX-minX, maxY-minY)) > maxVisInTiles){
      System.out.println("hypot "+dist);
      System.out.println("maxvi "+maxVisInTiles);
    }*/

    for(int i = minX; i <= maxX; i++){
      if(i < 0 || i >= obsMap.WIDTH)
        continue;
      for(int j = minY; j <= maxY; j++){
        if(j < 0 || j >= obsMap.HEIGHT)
          continue;
        ObserverTile tile = obsMap.getTile(i,j);
        obs.performTileCheck(tile, dt);
      }
    }
  }

  public void remove(ObserverModel obs){
    obs.deactivate();
    //throw new UnsupportedOperationException("Remove not implemented yet!");
    //Remove sight event update and remove from map
  }

  public EventQueue getObservationQueue(){
    return queue;
  }

}
