package jwars.model.weapons;

import jwars.data.ArmourTable;
import jwars.model.WarModel;
import jwars.model.units.Vehicle;
import java.util.Random;

public class VehicleArmourModel implements ArmourModel {

  private final ArmourTable hull, turret;
  private final static float TURRET_HIT_PROBABILITY = 0.3f;
  private Vehicle body;
  private float length = 6, width = 3; //implement proper numbers later
  //right now it's the same for hull and turret, for example

  public VehicleArmourModel(Vehicle body, ArmourTable hull, ArmourTable turret){
    this.body = body;
    this.hull = hull;
    this.turret = turret;
  }

  public void reportImpact(Weapon weapon, int impactX, int impactY,
                           float impactDist, float travellingRange,
                           float impactAngle){
    if(weapon.CAT.baseSteelPenetration == 0){
      return; //non-penetrating ammunition
    }
    //String hitLocation;
    if(impactDist < body.getRadius()){ //direct hit, not splash
      //System.out.println("Dist "+WarModel.WORLD.toMetres((int)impactDist));
      //System.out.println("Radius "+WarModel.WORLD.toMetres(body.getRadius()));
      Random rand = WarModel.WORLD.rand;
      ArmourTable armour;
      if(rand.nextFloat() < TURRET_HIT_PROBABILITY){ //decide whether hull or turret hit
        armour = turret;
        impactAngle -= body.getTurretAngle();
        //hitLocation = "turret";
      } else {
        //hitLocation = "hull";
        armour = hull;
      }

      float pen = (float)weapon.CAT.baseSteelPenetration;
      float cos = (float)Math.cos(impactAngle);
      float sin = (float)Math.sin(impactAngle);
      float absCos = Math.abs(cos);
      float absSin = Math.abs(sin);
      float sideHitProbability = length * absSin / (width * absCos + length * absSin);
      //System.out.println("side hit probability "+sideHitProbability);
      float armourThickness;

      if(WarModel.WORLD.rand.nextFloat() < sideHitProbability){ //side hit
        //divide by the sine instead(!), because the side is perpendicular
        armourThickness = armour.SIDE / absSin / (float)Math.cos(armour.ANGLE_SIDE);
        //hitLocation = "side "+hitLocation+" "+armour.SIDE+"mm@"+armour.ANGLE_SIDE;
      } else { //front or rear
        if(cos > 0){ //front
          armourThickness = armour.FRONT / absCos / (float)Math.cos(armour.ANGLE_FRONT);
          //hitLocation = "front "+hitLocation+" "+armour.FRONT+"mm@"+armour.ANGLE_FRONT;
        } else { //rear
          armourThickness = armour.REAR / absCos / (float)Math.cos(armour.ANGLE_REAR);
          //hitLocation = "rear "+hitLocation+" "+armour.REAR+"mm@"+armour.ANGLE_REAR;
        }
      }
      //System.out.println("---"+body+" "+body.getId()+" hit by "+weapon+", angle "+impactAngle+"---");
      //System.out.println("Hit location: "+hitLocation);
      //System.out.println("Eff armour: "+armourThickness+"\t Penetration: "+pen);
      if(pen > armourThickness){
        //System.out.println("Destroyed!");
        body.destroyLater();
      }// else {
      //  System.out.println("Shell bounces off");
      //}
    }
  }

  public float vulnerability(Weapon weapon, float range, float angle){
    return weapon.CAT.baseSteelPenetration > 0 ? 1 : 0;
    //later we can account for range, angle
  }
}
