package jwars.model.weapons;

import java.util.ArrayList;
import java.util.List;
import jwars.model.WarModel;

import jwars.data.WeaponCategory;

public class WeaponModel implements ReloadListener {

  private final List<Weapon> weapons = new ArrayList<Weapon>(0);
  private boolean canFire;
  private boolean mayFire = true;
  private int maxRange;
  //private ReloadListener listener;

  public WeaponModel(ReloadListener listener, List<WeaponCategory> weaponry){
    for(WeaponCategory cat : weaponry){
      Weapon w = cat.create();
      maxRange = (int)(Math.max(maxRange, cat.range));
      //this.listener = listener;
      w.setReloadListener(this);
      weapons.add(w);
    }
    canFire = isAtLeastOneWeaponReady();
  }

  private boolean isAtLeastOneWeaponReady(){
    for(Weapon w : weapons){
      if(w.isLoaded())
        return true;
    }
    return false;
  }

  public void fire(int srcX, int srcY, int dstX, int dstY){
    for(Weapon w : weapons){
      if(w.isLoaded()){
        w.fire(srcX, srcY, dstX, dstY);
      }
    }
    canFire = false;
  }

  public void setFireAtWill(boolean value){
    this.mayFire = value;
  }

  public boolean reportReloaded(Weapon weapon){
    canFire = true;
    return mayFire;//listener.reportReloaded(weapon);
  }

  public List<Weapon> getWeaponList(){
    return weapons;
  }

  public boolean canFire(){
    return canFire;
  }

  public int maxRange(){
    return maxRange;
  }

}
