package jwars.net;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;

public class Client{

  private boolean isListening = false;

  private final Protocol protocol;
  final ClientEventHandler eventHandler;

  public Client(Protocol protocol, ClientEventHandler eventHandler){
    this.protocol = protocol;
    if(protocol.client != null){
      throw new RuntimeException("Fixme: Protocol cannot be used by more than one client class");
    }
    protocol.client = this;
    this.eventHandler = eventHandler;
  }

  public void connect(String ip, int port) throws UnknownHostException, IOException {
    InetAddress host = InetAddress.getByName(ip);
    System.out.println("Client: Attempting to connect to "+host);
    Socket socket = new Socket(host, port);
    new ListeningThread(socket);
    System.out.println("Client: Connection established to "+socket);
  }

  public void fireClientPropertyChange(){
    try{
      protocol.clientPropChng.send(null);
    } catch(IOException ex){
      ex.printStackTrace();
    }
  }

  private class ListeningThread
      extends Thread{

    private final DataInputStream input;
    private final DataOutputStream output;

    private IOHandler currentHandler;
    private IOHandler lastHandler; //For error detection

    private final int clientIndex;

    public ListeningThread(Socket socket) throws IOException{
      super("Client Link Thread");
      this.input = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
      this.output = new DataOutputStream(new BufferedOutputStream(socket.getOutputStream()));
      this.clientIndex = input.readInt();
      protocol.setOut(output);
      eventHandler.handShake(clientIndex, input, output);
      processInstruction(input); //client join instruction regarding THIS client
      Client.this.isListening = true;
      start();
    }

    private synchronized void processInstruction(DataInputStream input) throws IOException {
      byte id = input.readByte();
      int timeStamp = input.readInt();
      currentHandler = protocol.getType(id);
      currentHandler.read(input, timeStamp);
      lastHandler = currentHandler;
    }

    public void run(){
      System.out.println("Client: Now listening");
      try{
        while(isListening){
          processInstruction(input);
        }
      }
      catch(SocketException ex){
        ex.printStackTrace();
      }
      catch(IOException ex){
        printErrorMsg(currentHandler, lastHandler);
        ex.printStackTrace();
      }
      catch(IndexOutOfBoundsException ex){
        printErrorMsg(currentHandler, lastHandler);
        ex.printStackTrace();
      }
    }

    private void printErrorMsg(IOHandler s1, IOHandler s2){
      System.err.println("Client: Exception thrown during I/O");
      System.err.println("Current handler: "+s1);
      System.err.println("Last handler: "+s2);
    }

  }

}
/**
 *   public void connect(String host){
    int response = Ref.empty;
    try{
      this.host = InetAddress.getByName(host);
      link = new Socket(this.host, PORT);
      in = new DataInputStream(link.getInputStream());
      out = new DataOutputStream(link.getOutputStream());
      response = in.read();
    } catch (IOException ex){
      System.out.println("<Client> Not Connected!");
      System.exit(3);
    }
    if (response == Ref.connected){
      connectionEstablished = true;
      System.out.println("<Client> Connection Established");
      ClientThread clt = new ClientThread(link, parser);
      clt.initialize();
    }
  }

 */
