package jwars.synch;

/**
 * By implementing this interface it will be possible to register with a Timer,
 * receiving regular update events.
 */
public interface Dynamical{


    /**
     * Do whatever might be appropriate.
     *
     * This method will be invoked each frame if the Dynamical is registered
     * with a Timer.

     * @param isLate whether the Timer is currently behind schedule
     */
    public void update(int frameCount, boolean isLate);

    /**
     * This method is invoked each frame by a Timer which is paused or
     * suspended. It may be used to adjust the Timer behaviour or otherwise
     * respond to the pause or suspension.
     */
    public void reportStasis();

}
