package jwars.test;

import com.shavenpuppy.jglib.algorithms.*;
import java.util.List;
import java.util.ArrayList;
import com.shavenpuppy.jglib.algorithms.PathFinder.State;
import jwars.model.map.TileMap;
import com.shavenpuppy.jglib.util.IntList;

public class PathFindTest{

  public static void main(String[] args){
    System.out.println("Init");
    PFMap map = new PFMap(20,30);
    PathFinderInt pf = new AStarInt(map, 100);
    IntList list = new IntList();
    System.out.println("Will find path:");
    int start = 3+2*map.width;
    int end = map.size-17-2*map.width;
    System.out.println("start "+map.stringXY(start));
    System.out.println("end "+map.stringXY(end));
    pf.findPath(3+2*map.width, map.size-17-2*map.width, list);
    int step;
    do{
      step = pf.nextStep();
      System.out.println(""+step);
    }
    while(step == pf.SEARCH_STATE_SEARCHING);
    if(step == pf.SEARCH_STATE_SUCCEEDED){
      System.out.println("Yay! Success!");
    } else {
      System.out.println("Monkeypants!");
    }
    for(int i = 0; i < list.size(); i++){
      //int point = list.get(i);
      //int x = map.getX(i);
      System.out.println(""+list.get(i));
    }
    System.out.println("Done.");
  }

  private static class PFMap implements PathFinderInt.Map {

    public final int width, height;
    public final int size;

    public PFMap(int width, int height){
      this.width = width;
      this.height = height;
      this.size = width*height;
    }

    public int getCost(int from, int to){
      int diff = Math.abs(to-from);
      if(diff == 1 || diff == width){ //not diagonal
        //System.out.println("cost from "+from+" to "+to);
        return 10;
      } else { //diagonal
        return 14;
      }
    }

    public void getNeighbours(int node, int parent, IntList dest){
      System.out.println("getting neighbours of "+stringXY(node));
      //dest.clear();
      dest.clear();
      int x = getX(node);
      int y = getY(node);
      possiblyAdd(index(x-1,y-1), parent, dest);
      possiblyAdd(index(x-1,y  ), parent, dest);
      possiblyAdd(index(x-1,y+1), parent, dest);
      possiblyAdd(index(x  ,y-1), parent, dest);
      possiblyAdd(index(x  ,y+1), parent, dest);
      possiblyAdd(index(x+1,y-1), parent, dest);
      possiblyAdd(index(x+1,y  ), parent, dest);
      possiblyAdd(index(x+1,y+1), parent, dest);
      System.out.println("---------------");
      System.out.println(""+this.intListToString(dest));
      System.out.println("---------------");
    }

    private String intListToString(IntList list){
      StringBuffer sb = new StringBuffer();
      for(int i = 0; i < list.size(); i++){
        sb.append("\n "+stringXY(list.get(i)));
      }
      return sb.toString();
    }

    private void possiblyAdd(int neighbour, int parent, IntList dest){
      if(contains(neighbour) /*&& neighbour != parent*/){
        dest.add(neighbour);
      }
    }

    public String stringXY(int index){
      return ""+getX(index)+","+getY(index);
    }

    public boolean contains(int index){
      return index >= 0 && index < size;
    }

    public int getDistance(int from, int to){
      int x1 = getX(from);
      int y1 = getY(from);
      int x2 = getX(to);
      int y2 = getY(to);
      return (int)(100*Math.hypot(x2-x1,y2-y1));
    }

    public int index(int x, int y){
      return x + y*width;
    }

    public int getX(int index){
      return index % width;
    }

    public int getY(int index){
      return index / width;
    }

  }



}
